require "import"
import "android.widget.*"
import "com.androlua.*"
import "android.content.*"
import "java.util.Locale"
import "android.content.ClipboardManager"
import "android.content.ClipData"
import "android.content.Intent"
import "android.view.View"

-- Check if the device language is Thai
local isThai = Locale.getDefault().getLanguage() == "th"

-- Localized strings
local strings = {
  title = isThai and "แยกและแชร์ลิงก์" or "Extract and Share Links",
  inputLabel = isThai and "วางข้อความของคุณที่นี่:" or "Paste your text here:",
  extractButton = isThai and "แยก" or "Extract",
  shareButton = isThai and "แชร์" or "Share",
  copyButton = isThai and "คัดลอก" or "Copy",
  clearButton = isThai and "ล้าง" or "Clear",
  howToButton = isThai and "วิธีใช้" or "How to Use",
  closeButton = isThai and "ปิด" or "Close",
  noLinksFound = isThai and "ไม่พบลิงก์ในข้อความ" or "No links found in the text",
  linksFound = isThai and "พบลิงก์ %d ลิงก์:" or "Found %d link(s):",
  shareVia = isThai and "แชร์ลิงก์ผ่าน:" or "Share link via:",
  noLinksToShare = isThai and "ไม่พบลิงก์ที่จะแชร์" or "No links to share",
  linkCopied = isThai and "คัดลอกลิงก์แล้ว" or "Link copied",
  howToUseTitle = isThai and "วิธีใช้งาน" or "How to Use",
  howToUseContent = isThai and "1. คัดลอกข้อความที่มีลิงก์\n2. เปิดแอพนี้\n3. ข้อความจะถูกวางโดยอัตโนมัติและลิงก์จะถูกแยก\n4. คุณสามารถคัดลอกหรือแชร์ลิงก์แต่ละอัน\n5. กดปุ่ม 'ล้าง' เพื่อเริ่มใหม่" 
    or "1. Copy text containing links\n2. Open this app\n3. Text will be pasted automatically and links will be extracted\n4. You can copy or share individual links\n5. Press 'Clear' to start over",
  shareWithMessageButton = isThai and "แชร์พร้อมข้อความ" or "Share with Message",
  enterMessagePrompt = isThai and "กรุณาใส่ข้อความ:" or "Enter your message:",
  ok = isThai and "ตกลง" or "OK",
  cancel = isThai and "ยกเลิก" or "Cancel"
}

-- Function to extract links from text (improved)
local function extractLinks(text)
  local links = {}
  for link in text:gmatch("https?://[%w%.%-%+_#=/%?&%%]+") do
    table.insert(links, link)
  end
  if #links == 0 then
    for link in text:gmatch("www%.[%w%.%-%+_#=/%?&%%]+") do
      table.insert(links, "http://" .. link)
    end
  end
  return links
end

-- Function to copy text to clipboard
local function copyToClipboard(text)
  local clipboard = this.getSystemService(Context.CLIPBOARD_SERVICE)
  local clip = ClipData.newPlainText("link", text)
  clipboard.setPrimaryClip(clip)
  Toast.makeText(this, strings.linkCopied, Toast.LENGTH_SHORT).show()
end

-- Function to share a single link
local function shareLink(link, message)
  local sharingIntent = Intent(Intent.ACTION_SEND)
  sharingIntent.setType("text/plain")
  local shareText = message and (message .. "\n\n" .. link) or link
  sharingIntent.putExtra(Intent.EXTRA_TEXT, shareText)
  this.startActivity(Intent.createChooser(sharingIntent, strings.shareVia))
end

-- Function to show share options
local function showShareOptions(link)
  local shareOptionsLayout = {
    LinearLayout;
    orientation="vertical";
    Gravity="center";
    {
      Button;
      text=strings.shareButton;
      layout_width="fill";
      onClick=function() shareLink(link) end;
    };
    {
      Button;
      text=strings.shareWithMessageButton;
      layout_width="fill";
      onClick=function()
        local messageInputLayout = {
          LinearLayout;
          orientation="vertical";
          Gravity="center";
          {
            EditText;
            hint=strings.enterMessagePrompt;
            layout_width="fill";
            id="messageInput";
          };
        }
        local messageDialog = LuaDialog(this)
        messageDialog.setTitle(strings.enterMessagePrompt)
        messageDialog.setView(loadlayout(messageInputLayout))
        messageDialog.setPositiveButton(strings.ok, function()
          local message = messageInput.Text
          shareLink(link, message)
        end)
        messageDialog.setNegativeButton(strings.cancel, nil)
        messageDialog.show()
      end;
    };
  }
  
  local shareDialog = LuaDialog(this)
  shareDialog.setTitle(strings.shareVia)
  shareDialog.setView(loadlayout(shareOptionsLayout))
  shareDialog.setNegativeButton(strings.cancel, nil)
  shareDialog.show()
end

-- Function to display extracted links
local function displayLinks(links, layout)
  layout.removeAllViews()
  if #links > 0 then
    local message = string.format(strings.linksFound, #links)
    local headerView = TextView()
    headerView.setText(message)
    headerView.setTextSize(18)
    headerView.setPadding(20, 20, 20, 20)
    layout.addView(headerView)
    for i, link in ipairs(links) do
      local linkLayout = LinearLayout()
      linkLayout.setOrientation(LinearLayout.HORIZONTAL)
      
      local linkView = TextView()
      linkView.setText(link)
      linkView.setTextSize(16)
      linkView.setPadding(20, 10, 20, 10)
      linkView.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT, 1))
      linkLayout.addView(linkView)
      
      local copyButton = Button(this)
      copyButton.setText(strings.copyButton)
      copyButton.setOnClickListener{
        onClick = function()
          copyToClipboard(link)
        end
      }
      linkLayout.addView(copyButton)
      
      local shareButton = Button(this)
      shareButton.setText(strings.shareButton)
      shareButton.setOnClickListener{
        onClick = function()
          showShareOptions(link)
        end
      }
      linkLayout.addView(shareButton)
      
      layout.addView(linkLayout)
    end
  else
    local noLinksView = TextView()
    noLinksView.setText(strings.noLinksFound)
    noLinksView.setTextSize(18)
    noLinksView.setPadding(20, 20, 20, 20)
    layout.addView(noLinksView)
  end
end

-- Create UI elements
local inputLayout = {
  LinearLayout;
  orientation = "vertical",
  layout_width = "fill",
  {
    TextView;
    text = strings.inputLabel,
    textSize = 18,
    textColor = 0xff000000,
    layout_width = "wrap",
    layout_marginTop = "10dp",
    layout_marginLeft = "10dp",
    layout_marginRight = "10dp"
  },
  {
    EditText;
    id = "textInput",
    layout_width = "fill",
    layout_height = "150dp",
    gravity = "top",
    layout_marginLeft = "10dp",
    layout_marginRight = "10dp"
  },
  {
    LinearLayout;
    orientation = "horizontal",
    layout_width = "fill",
    {
      Button;
      text = strings.extractButton,
      id = "extractButton",
      layout_weight = 1,
      layout_margin = "5dp"
    },
    {
      Button;
      text = strings.clearButton,
      id = "clearButton",
      layout_weight = 1,
      layout_margin = "5dp"
    },
    {
      Button;
      text = strings.howToButton,
      id = "howToButton",
      layout_weight = 1,
      layout_margin = "5dp"
    }
  },
  {
    TextView;
    id = "howToTextView",
    text = strings.howToUseContent,
    textSize = 14,
    padding = "10dp",
    visibility = View.GONE
  },
  {
    ScrollView;
    layout_width = "fill",
    layout_height = "fill",
    {
      LinearLayout;
      id = "linksLayout",
      orientation = "vertical",
      layout_width = "fill"
    }
  }
}

-- Create dialog
local dlg = LuaDialog()
dlg.setTitle(strings.title)
dlg.setView(loadlayout(inputLayout))

-- Function to process clipboard content
local function processClipboard()
  local clipboard = this.getSystemService(Context.CLIPBOARD_SERVICE)
  if clipboard.hasPrimaryClip() then
    local item = clipboard.getPrimaryClip().getItemAt(0)
    local pastedText = item.getText()
    if pastedText then
      textInput.setText(pastedText)
      local links = extractLinks(pastedText)
      displayLinks(links, linksLayout)
    end
  end
end

-- Set up extract button functionality
extractButton.onClick = function()
  local text = textInput.text
  local links = extractLinks(text)
  displayLinks(links, linksLayout)
end

-- Set up clear button functionality
clearButton.onClick = function()
  textInput.setText("")
  linksLayout.removeAllViews()
  howToTextView.setVisibility(View.GONE)
end

-- Set up how-to button functionality
howToButton.onClick = function()
  if howToTextView.getVisibility() == View.GONE then
    howToTextView.setVisibility(View.VISIBLE)
  else
    howToTextView.setVisibility(View.GONE)
  end
end

-- Set the close button
dlg.setButton3(strings.closeButton, function()
  dlg.dismiss()
end)

-- Process clipboard content when the dialog is shown
dlg.setOnShowListener{
  onShow = function()
    processClipboard()
  end
}

dlg.show()